using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.OleDb;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using RealLife1.DbDemosTableAdapters;
using System.Transactions;

namespace RealLife1
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }



        /// <summary>
        /// Chargement de la fiche
        /// </summary>
        /// <param name="sender">Source</param>
        /// <param name="e"><see cref="System.EventArgs"/> instance</param>
        private void Form1_Load(object sender, EventArgs e)
        {
            // Chargement de la table lookup des pays
            countryTableAdapter.Fill(countryDataSet.country);
            // Chargement des clients
            LoadCustomerData();
        }

        private bool _LoadingCustomer;

        /// <summary>
        /// Charge les clients.
        /// Utilise la textbox de recherche pour filtrer les donnes
        /// </summary>
        private void LoadCustomerData()
        {
            _LoadingCustomer = true;
            try
            {
                // il faut vider les tables lies sinon une erreur de violation
                // de contrainte sera leve. En effet les commandes de "orders"
                // n'auront plus forcment de client correspondant dans "customer" 
                // la mme logique s'applique  la table "items" vis  vis de "orders"
                // L'ordre de vidage est donc l'inverse de celui de la descente des cls.
                dbDemos1.items.Clear();
                dbDemos1.orders.Clear();
                // la commande SELECT possde un paramtre qui filtre la liste
                // pour limiter le nombre de records remonts en mmoire.
                customerTableAdapter.Fill(dbDemos1.customer, tbSearchCompany.Text.Trim());
                // affichage du nombre de clients remonts
                int i = dbDemos1.customer.Count;
                lCount.Text = i + " client" + (i > 1 ? "s" : "");
                // RAZ du flag indiquant si le record en cours est modifi ou non
                CustomerModified = false;
                // int custno = (int)CustomersGrid.SelectedRows[0].Cells["CustNo"].Value;
                // ordersTableAdapter.Fill(dbDemos1.orders,custno);
                //itemsTableAdapter.Fill(dbDemos1.items);            
            }
            finally { _LoadingCustomer = false; }
        }

        /// <summary>
        /// Charge les commandes du client en cours.
        /// </summary>
        private void LoadOrdersData()
        {
           ordersTableAdapter.Fill(dbDemos1.orders, Convert.ToInt32(lCustNo.Text));
        }

        private bool _CustomerModified;
        /// <summary>
        /// Proprit indiquant l'tat du record en cours (modifi ou non)
        /// </summary>
        /// <value><c>true</c> modifi; sinon, <c>false</c>.</value>
        private bool CustomerModified
        {
            get
            {
                return _CustomerModified;
            }
            set
            {
                _CustomerModified = value;
                // On rpercute l'tat sur les lments visuels
                btnUpdateCustomer.Enabled = value; // bouton MAJ
                btnCancelCustomerChanges.Enabled = value; // bouton annuler
                CustomersGrid.Enabled = !value; // navigation dans la grille on/off
                btnFindCustomer.Enabled = !value; // bouton de recherche
            }
        }


        /// <summary>
        /// Clic sur le bouton de fermeture de l'application
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void close_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        /// <summary>
        /// Applique la mise  jour des clients sur la base de donnes
        /// </summary>
        private void UpdateCustomer()
        {
            customerBindingSource.EndEdit();
            try
            {
                customerTableAdapter.Update(dbDemos1.customer);
                CustomerModified = false;
            }
            catch (DBConcurrencyException)
            {
                MessageBox.Show("Impossible de mettre  jour la fiche (accs concurrents). Les donnes vont tre recharges.");
                int currentCustomer = Convert.ToInt32(CustomersGrid.SelectedRows[0].Cells["CustNo"].Value);
                LoadCustomerData();
                int newpos = -1;
                for (int i=0;i<CustomersGrid.RowCount;i++)
                {
                  if (Convert.ToInt32(CustomersGrid.Rows[i].Cells["CustNo"].Value) == currentCustomer)
                  {
                      newpos = i; break;
                  }
                }
                if (newpos > 1) CustomersGrid.CurrentCell = CustomersGrid.Rows[newpos].Cells[0];
            }
        }


        /// <summary>
        /// Clic sur le bouton de recherche des clients
        /// </summary>
        /// <param name="sender">le bouton</param>
        /// <param name="e">arguments</param>
        private void btnFindCustomer_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(tbSearchCompany.Text.Trim()))
            {
                MessageBox.Show("Vous devez taper les premires lettres du client recherch !");
                return;
            }
            // simple appel au chargement des donnes puisque le textbox
            // de recherche est systmatiquement utilis...
            LoadCustomerData();
        }

        /// <summary>
        /// Gestionnaire de l'vnement Changed des contols servant  modifier
        /// les informations du client affich.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CustomerDataChanged(object sender, EventArgs e)
        {
            // on indique par le drapeau que le record en cours est modifi
            // ce qui dclenche l'ajustement de l'interface (boutons disabled, etc)
            if (!_LoadingCustomer) CustomerModified = true;
        }

        /// <summary>
        /// La position dans la binding source des clients a chang
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void customerBindingSource_PositionChanged(object sender, EventArgs e)
        {
            // on annule le drapeau de modification lev  tort par le changement de
            // ligne qui a entran le changement de contenu des textbox, donc la leve
            // du drapeau  true.
            if (!_LoadingCustomer) CustomerModified = false;
            //CustomerModified =
            //    ((CustomersGrid.SelectedRows.Count > 0) && (CustomersGrid.SelectedRows[0].Cells["CustNo"].Value != null)) ?
            //    dbDemos1.customer.Select("CustNo=" + CustomersGrid.SelectedRows[0].Cells["CustNo"].Value.ToString())[0].
            //        RowState !=
            //        DataRowState.Unchanged : false;
        }

        /// <summary>
        /// Clic sur le bouton de mise  jour des clients
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnUpdateCustomer_Click(object sender, EventArgs e)
        {
            // Appel  la mthode d'update
            UpdateCustomer();
        }

        /// <summary>
        /// Clic sur le bouton d'annulation des modifications du client en cours
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCancelCustomerChanges_Click(object sender, EventArgs e)
        {
            // Annulation du mode d'dition de la binding source
            customerBindingSource.CancelEdit();
            // Annulation du drapeau 
            CustomerModified = false;
        }

        /// <summary>
        /// Changement de page du tabcontrol.
        /// Chargement des donnes selon la page.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tabControl1_Selected(object sender, TabControlEventArgs e)
        {
            if (e.TabPageIndex == 1) LoadOrdersData();
        }

        /// <summary>
        /// Double clic grille des clients.
        /// Ouvre la page des commandes.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CustomersGrid_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            tabControl1.SelectedIndex = 1;
        }

    }
}